
#pragma once

#include <Base_Provider.h>
#include <unordered_map>
#include <hiredis/hiredis.h>
#include <Base_Configurator.h>

namespace inaf::oasbo::Providers {

/**
 * @brief The RedisProvider class is a concrete implementation of the BaseProvider class.
 * It provides functionality to write data to a Redis server.
 * Check Base_Provider.h for more information.
 */
class RedisProvider: public BaseProvider {
protected:
	redisContext *context = nullptr;

	/**
	 * @brief Default constructor for RedisProvider.
	 */
	RedisProvider();

	/**
	 * @brief Constructor for RedisProvider.
	 * @param ip The IP address of the Redis server.
	 * @param port The port number of the Redis server.
	 * @param key The key used to access the Redis server.
	 */
	RedisProvider(std::string ip, int port, std::string key);

public:
	std::string ip; /**< The IP address of the Redis server. */
	int port; /**< The port number of the Redis server. */
	std::string key; /**< The key used to access the Redis server. */

	int write(Packets::BasePacket&) override;

	int write(Packets::BasePacket&, std::string dest) override;

	void setDest(std::string dest) override;

	std::string getDest() override;

	int close() override;

	int open() override;

	bool isOpen() override;

	/**
	 * @brief Destructor for RedisProvider.
	 */
	~RedisProvider();

	friend class RedisProviderBuilder;
};

/**
 * @brief The RedisProviderBuilder class is used to build RedisProvider objects.
 */
class RedisProviderBuilder {
protected:
	RedisProvider *provider;

public:
	std::string config_target { "redisprovider" };
	std::string ip_key { "ip" };
	std::string port_key { "port" };
	std::string redis_key_key { "key" };

	/**
	 * @brief Default constructor for RedisProviderBuilder.
	 */
	RedisProviderBuilder();

	/**
	 * @brief Constructor for RedisProviderBuilder.
	 * @param ip The IP address of the Redis server.
	 * @param port The port number of the Redis server.
	 * @param topic The topic to write packets to.
	 */
	RedisProviderBuilder(std::string ip, int port, std::string topic);

	/**
	 * @brief Destructor for RedisProviderBuilder.
	 */
	~RedisProviderBuilder();

	/**
	 * @brief Resets the builder to its initial state.
	 */
	void reset();

	/**
	 * @brief Configures the builder from a BaseConfigurator object.
	 * @param conf The BaseConfigurator object to configure from.
	 * @return A pointer to the RedisProviderBuilder object.
	 */
	RedisProviderBuilder* configFrom(Configurators::BaseConfigurator &conf);

	/**
	 * @brief Sets the IP address of the Redis server.
	 * @param ip The IP address to set.
	 * @return A pointer to the RedisProviderBuilder object.
	 */
	RedisProviderBuilder* setIp(std::string ip);

	/**
	 * @brief Sets the port number of the Redis server.
	 * @param port The port number to set.
	 * @return A pointer to the RedisProviderBuilder object.
	 */
	RedisProviderBuilder* setPort(int port);

	/**
	 * @brief Sets the key used to access the Redis server.
	 * @param key The key to set.
	 * @return A pointer to the RedisProviderBuilder object.
	 */
	RedisProviderBuilder* setKey(std::string key);

	/**
	 * @brief Gets the built RedisProvider object.
	 * @return A pointer to the built RedisProvider object.
	 */
	RedisProvider* getProvider();
};

} // namespace inaf::oasbo::Providers
